<?php


// Attempt to hijack data from the acu-rite bridge.  The bridge makes
// POST requests to www.acu-link.com/messages. That name has been redirected
// here. 
//
// This tries to consolidate the raw data into a 1 minute consolidated
// packet that would be suitable for writing to a database. It does this
// by saving the data in a temporary file with counts and sums, then at
// the start of a new minute, it processes the data in the file to create
// average values.
//
// Currently the data packet is written to 'raw.txt' every minute.
//
// In addition, send the raw data to the acu-link server so that MBW and
// weather underground sharing continue to work.

date_default_timezone_set('America/Los_Angeles');
$ts = new DateTime('NOW');

// Make sure we're getting the a POST request
if (!empty($_POST)) {
	
	$datastore = array (
		"timestamp" => -1,
		"t_cnt" => 0,
		"h_cnt" => 0,
		"s_cnt" => 0,
		"p_cnt" => 0,
		"d_cnt" => 0,
		"temperature" => 0,
		"humidity" => 0,
		"windspeed" => 0,
		"pressure" => 0,
		"winddirection" => 0,
		"rainfall" => 0,
		"sensor" => 0,
	);

	// Get the saved data from the temporary file
	$datastore = get_saved_data();

	if ($datastore['timestamp'] != date("i")) {
		// If this is the start of a new minute, process the existing
		// data in the temporary file and then clear it.
		process_data($datastore);

		// clear data
		$datastore['t_cnt'] = 0;
		$datastore['temperature'] = 0;
		$datastore['h_cnt'] = 0;
		$datastore['humidity'] = 0;
		$datastore['s_cnt'] = 0;
		$datastore['windspeed'] = 0;
		$datastore['p_cnt'] = 0;
		$datastore['pressure'] = 0;
		$datastore['d_cnt'] = 0;
		$datastore['winddirection'] = 0;
	   	$datastore['rainfall'] = 0;
	   	$datastore['sensor'] = 0;
	}

	$datastore['timestamp'] = date("i");
	$repost = "";  // String to send to MBW

	$fh = fopen('test.txt', 'a');

	$sensor = $_POST["sensor"];

	// Parse each of the key=value pairs of the POST data.
	foreach ($_POST as $ky => $value) {
		$repost = $repost . "&" . $ky . "=" . $value;

		switch ($ky) {
		case "mt":
			$name = $value;
			break;
		case "battery":
			$battery = $value;
			break;
		case "temperature":
			sscanf($value, "A0%02d%d", $a, $b);
			$t = $a . "." . $b;
			$temp = floatval($t) * 1.8 + 32;
			// Temperature data from the 5n1
			if ($sensor == "01133") {
				$datastore['temperature'] += $temp;
				$datastore['t_cnt']++;
			}
			break;
		case "humidity":
			// humidity = A0230
			sscanf($value, "A0%02d%d", $a, $b);
			$humidity = $a . "." . $b;
			// Use the tower sensors's humidity value since it's right
			if ($sensor == "01274") {
				$datastore['humidity'] += $humidity;
				$datastore['h_cnt']++;
			}
			break;
		case "windspeed":
			// windspeed = A000000000
			sscanf($value, "A0%02d%d", $a, $b);
			$speed = $a . "." . $b;
			// Windspeed data from the 5n1
			if ($sensor == "01133") {
				$datastore['windspeed'] += $speed;
				$datastore['s_cnt']++;
			}
			break;
		case "winddir":
			// This is a hex digit representing the direction?
			// 0 = north
			// 4 = west
			// 8 = south
			// c = east
			$direction = hexdec($value);
			$datastore['winddirection'] += $direction;
			$datastore['d_cnt']++;
			break;
		case "rainfall":
			sscanf($value, "A0%02d%d", $a, $b);
			$rain = $a . "." . $b;
			// rainfall data from the 5n1
			if ($sensor == "01133") {
				$datastore['rainfall'] += $rain;
			}
			break;
		case "PR":  // Pressure value
			//  For 29.88 hg or 1011.84892768 millibars
			// C1 = 4B8D = 19341
			// C2 = 0CD3 = 3283
			// C3 = 0135 = 309
			// C4 = 0257 = 599
			// C5 = 826D = 33389
			// C6 = 17CA = 6090
			// C7 = 09C4 = 2500
			// A = 07
			// B = 2A = 42
			// C = 06
			// D = 09
			// PR = 8FD0 = 36816
			// TR = 81DF = 33247
			//
			// It looks like C1 - C7 are constant calibration values.
			// PR is the current raw pressure value (14 bits?)
			// TR is the current raw temperature value
			//
			// Two possible devices that seem to output similar types of values:
			// EPOCOS T5400 - This has C1 - C8, a PR and TR value
			// MS5535C - This has C1 - C6, D1, D2 values
			// The EPOCOS T5300 sounds like it could be close but not info?
			$pressure = $value;
			$datastore['sensor'] = $pressure;
			$datastore['pressure'] += hexdec($pressure & 0x3fff);
			$datastore['p_cnt']++;
			break;
		case "TR":  // Temperature value
			break;
		}
   	}

	// Save the data in the temporary file
	set_saved_data($datastore);

} else {
	// Not a POST request, do nothing
}

// Let the bridge know we got the data data
echo "SUCCESS";

//
// How should the data be saved?
//
// Database keyed on date (to minute). This might be best done by having
// an intermeediate database where we store counts and totals. When data
// comes in for the next minute, process the intermediate database and create
// an entry for the main database.
//
// Intermediate (which could just be a file) looks like this:
//
// <timestamp> <h_cnt> <h_total> <t_cnt> <t_total> <s_cnt> <s_total> <p_cnt> <p_total> <d_cnt> <d_total> <rainfall>
//
// timestamp could be just a minute value
// rainfall is just a total.
//
// read intermdiate
// if timestamp != current
//   process data (calculate averages and write to database)
//   clear data
//
// based on data, update values
// write intermediate
 

// Repost the data to MBW
$url = 'http://acu-link.com/messages';
$ch = curl_init($url);
curl_setopt($ch, CURLOPT_POST, 1);
curl_setopt($ch, CURLOPT_POSTFIELDS, $repost);
curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
curl_setopt($ch, CURLOPT_HEADER, 0);
curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
$response = curl_exec($ch);



// ************************************************************************
//
// Functions to save, get, and process the temporary file

function get_saved_data()
{
	$ds = fopen('datastore.txt', 'r');
	$data = fread($ds, filesize('datastore.txt'));

	// split data by '|' character and assign to datastore.
	$elements = explode("|", $data);
	$datastore['timestamp'] = $elements[0];
	$datastore['t_cnt'] = $elements[1];
	$datastore['temperature'] = $elements[2];
	$datastore['h_cnt'] = $elements[3];
	$datastore['humidity'] = $elements[4];
	$datastore['s_cnt'] = $elements[5];
	$datastore['windspeed'] = $elements[6];
	$datastore['p_cnt'] = $elements[7];
	$datastore['pressure'] = $elements[8];
	$datastore['d_cnt'] = $elements[9];
	$datastore['winddirection'] = $elements[10];
	$datastore['rainfall'] = $elements[11];
	$datastore['sensor'] = $elements[12];

	fclose($ds);

	return $datastore;
}

function set_saved_data($datastore)
{
	$ds = fopen('datastore.txt', 'w');
	fwrite($ds, $datastore['timestamp'] . "|");
	fwrite($ds, $datastore['t_cnt'] . "|");
	fwrite($ds, $datastore['temperature'] . "|");
	fwrite($ds, $datastore['h_cnt'] . "|");
	fwrite($ds, $datastore['humidity'] . "|");
	fwrite($ds, $datastore['s_cnt'] . "|");
	fwrite($ds, $datastore['windspeed'] . "|");
	fwrite($ds, $datastore['p_cnt'] . "|");
	fwrite($ds, $datastore['pressure'] . "|");
	fwrite($ds, $datastore['d_cnt'] . "|");
	fwrite($ds, $datastore['winddirection'] . "|");
	fwrite($ds, $datastore['rainfall'] . "|");
	fwrite($ds, $datastore['sensor']);
	fclose($ds);
}

function process_data ($ds)
{
	$fh = fopen('raw.txt', 'a');
	fwrite($fh, date("YmdHis") . ": ");
	if ($ds['t_cnt'] != 0) {
	   	fwrite($fh, $ds['temperature'] / $ds['t_cnt'] . " | ");
	} else {
	   	fwrite($fh, "0 | ");
	}

	if ($ds['h_cnt'] != 0) {
	   	fwrite($fh, $ds['humidity'] / $ds['h_cnt'] . "% | ");
	} else {
	   	fwrite($fh, "0% | ");
	}

	if ($ds['p_cnt'] != 0) {
	   	fwrite($fh, $ds['pressure'] / $ds['p_cnt'] . " mb | ");
	} else {
	   	fwrite($fh, "0 mb | ");
	}

	if ($ds['s_cnt'] != 0) {
	   	fwrite($fh, $ds['windspeed'] / $ds['s_cnt'] . " mph | ");
	} else {
	   	fwrite($fh, "0 mph | ");
	}

	if ($ds['d_cnt'] != 0) {
	   	fwrite($fh, $ds['winddirection'] / $ds['d_cnt'] . " | ");
	} else {
	   	fwrite($fh, "0 | ");
	}

	fwrite($fh, $ds['rainfall'] . " in");
	fwrite($fh, " --> " . $ds['sensor'] );

	fwrite($fh, "\n");

	fclose($fh);
}

?>

